#!/usr/bin/python
#
"""Port Controler using UPnP.

usage: portctrl [-d url] [-c ppp/ip] [-t num] [-u num] ...
options:
    -d url: device XML url
    -c ppp/ip: connection type
    -t num: TCP port
    -u num: TCP port
"""
#
# Copyright (c) 2005 Satoshi Fukutomi <info@fuktommy.com>.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#

import sys
from time import sleep
from threading import Thread

from upnp import *
from portopener import *

upnp_timeout = 10       # Seconds
upnp_cycle = 1*60*60    # Seconds

def usage():
    sys.exit("usage: portctrl [-d url] [-c ppp/ip] [-t num] [-u num]")

def openport():
    argv = sys.argv[1:]
    ports = []
    xmlurl = ""
    contype = ""
    try:
        while argv:
            i = argv.pop(0)
            if i == "-d":
                xmlurl = argv.pop(0)
            elif i == "-c":
                contype = argv.pop(0)
            elif i == "-t":
                ports.append((int(argv.pop(0)), "tcp"))
            elif i == "-u":
                ports.append((int(argv.pop(0)), "udp"))
            else:
                usage()
    except IndexError, ValueError:
        usage()
    if not ports:
        usage()

    if xmlurl and contype:
        if contype == "ppp":
            contype = "WANPPPConnection"
        elif contype == "ip":
            contype = "WANIPConnection"
        router = Router(xmlurl, contype, upnp_timeout)
    else:
        router = findrouter(upnp_timeout)

    if not router:
        sys.exit("router nof found.")
    else:
        print "Router found: %s." % router

    openports = []
    for port, protocol in ports:
        flag = router.openport(port, protocol, "%s%d" % (protocol, port))
        if flag:
            print "Open port %d/%s succeeded." % (port, protocol)
            openports.append((port, protocol))
            p = PortOpener(port, protocol,
                           router=router,
                           openports=openports,
                           logger=Logger(),
                           cycle=upnp_cycle)
            p.start()
        else:
            print "Open port %d/%s failed." % (port, protocol)
            return router, openports
    return router, openports

def main():
    try:
        try:
            router, openports = openport()
            while True:
                sleep(upnp_cycle)
        finally:
            for port, protocol in openports:
                flag = router.closeport(port, protocol)
                if flag:
                    print "Close port %d/%s succeeded." % (port, protocol)
                else:
                    print "Close port %d/%s failed." % (port, protocol)
    except KeyboardInterrupt:
        sys.exit()

if __name__ == "__main__":
    main()
