#
"""Port Controler using UPnP.
"""
#
# Copyright (c) 2005 Satoshi Fukutomi <info@fuktommy.com>.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#

from time import sleep
from threading import Thread

__all__ = ["PortOpener", "Logger"]

upnp_cycle = 1*60*60    # Seconds


class PortOpener(Thread):
    def __init__(self, port, protocol, router=None,
                 openports=None, logger=None, cycle=upnp_cycle):
        Thread.__init__(self)
        self.port = port
        self.protocol = protocol
        self.router = router
        self.openports = openports
        self.logger = logger
        self.cycle = cycle
        self.setDaemon(True)

    def run(self):
        port = self.port
        protocol = self.protocol
        router = self.router
        logger = self.logger
        while True:
            sleep(self.cycle)
            if (self.port, self.protocol) not in self.openports:
                break
            flag = router.openport(port, protocol, "%s%d" % (protocol, port))
            if flag:
                logger.send("Open port %d/%s succeeded." % (port, protocol))
            else:
                logger.send("Open port %d/%s failed." % (port, protocol))


class Logger:
    def send(self, msg):
        print msg
